package solatium.blockchain;

import org.junit.Test;

import java.util.Date;
import java.util.List;

import static org.junit.Assert.*;

public class ChainTest {

    @Test
    public void shouldAddToChain() {
        String hash = "foo123";
        Transaction transaction = new Transaction("0", "0", 0);
        Block block = new TransactionBlock("", transaction, new Date().getTime());
        HashedBlock hashedBlock = new HashedBlock(block, hash);
        Chain chain = new Chain();
        chain.addToChain(hashedBlock);
        assertEquals(hash, chain.getLastHash());
    }

    @Test
    public void shouldValidateChain() {
        Chain chain = new Chain();
        for (int i = 0; i < 10; i++) {
            String previousHash = chain.getLastHash();
            chain.addToChain(getHashedBlock(previousHash));
        }
        assertTrue(chain.validateChain());
    }

    @Test
    public void shouldFailValidateChain() {
        Transaction transaction = new Transaction("0", "0", 0);
        Block wrongBlock = new TransactionBlock("", transaction, new Date().getTime());
        Chain chain = new Chain();
        for (int i = 0; i < 10; i++) {
            String previousHash = chain.getLastHash();
            chain.addToChain(getHashedBlock(previousHash));
        }
        chain.addToChain(wrongBlock.getHashed());
        assertFalse(chain.validateChain());
    }

    @Test
    public void shouldReturnListWithBlocks() {
        Chain chain = new Chain();
        Transaction transaction = new Transaction("fff", "xxx", 10);
        Block block = new TransactionBlock(chain.getLastHash(), transaction, new Date().getTime());
        chain.addToChain(block.getHashed());
        List<HashedBlock> hashedBlocks = chain.getBlockChain();
        assertEquals(2, hashedBlocks.size());
        assertEquals("Transaction{senderName='0', receiverName='0', value=0}", hashedBlocks.get(0).getBlock().getData().toString());
        assertEquals("Transaction{senderName='fff', receiverName='xxx', value=10}", hashedBlocks.get(1).getBlock().getData().toString());
    }

    @Test
    public void shouldToString() {
        Chain chain = new Chain();
        Transaction transaction = new Transaction("0", "0", 0);
        Block block0 = new TransactionBlock(chain.getLastHash(), transaction, 1);
        Block block1 = new TransactionBlock(chain.getLastHash(), transaction, 2);
        Block block2 = new TransactionBlock(chain.getLastHash(), transaction, 3);
        chain.addToChain(block0.getHashed());
        chain.addToChain(block1.getHashed());
        chain.addToChain(block2.getHashed());
        assertEquals("Chain{\n" +
                "HashedBlock{hash='2B0A03DE90CC5AAEE0AF05ABC1B68739416AB185B822A4B54586CEABCD9CE18A', block=Block{previousHash='0', data='Transaction{senderName='0', receiverName='0', value=0}', timestamp=0}}\n" +
                "HashedBlock{hash='54AA7114A20D2C6B0A0AC9582FC9427D68F17C1DB0919C63F61FE8A5EA06A344', block=Block{previousHash='2B0A03DE90CC5AAEE0AF05ABC1B68739416AB185B822A4B54586CEABCD9CE18A', data='Transaction{senderName='0', receiverName='0', value=0}', timestamp=1}}\n" +
                "HashedBlock{hash='4F3D56D9A87A64382D36FFD9FFBB216CDE1B543BC9195F31EC6D5EAF3F3AF2AF', block=Block{previousHash='2B0A03DE90CC5AAEE0AF05ABC1B68739416AB185B822A4B54586CEABCD9CE18A', data='Transaction{senderName='0', receiverName='0', value=0}', timestamp=2}}\n" +
                "HashedBlock{hash='6938106F7D731217E044E6051F95A132A29B255F6C1733D4C0AF5F077BBBA686', block=Block{previousHash='2B0A03DE90CC5AAEE0AF05ABC1B68739416AB185B822A4B54586CEABCD9CE18A', data='Transaction{senderName='0', receiverName='0', value=0}', timestamp=3}}\n" +
                "}", chain.toString());

    }

    private HashedBlock getHashedBlock(String previousHash) {
        Transaction transaction = new Transaction("0", "0", 0);
        Block block = new TransactionBlock(previousHash, transaction, new Date().getTime());
        return block.getHashed();
    }
}
